package dev.fyloz.colorrecipesexplorer.utils

import dev.fyloz.colorrecipesexplorer.JavaFile
import java.nio.file.Files
import java.nio.file.Path

/** Mockable file wrapper, to prevent issues when mocking [java.io.File]. */
class File(val file: JavaFile) {
    val isFile: Boolean
        get() = file.isFile

    fun toPath(): Path =
            file.toPath()

    fun exists() =
            file.exists()

    fun readBytes() =
            file.readBytes()

    fun writeBytes(array: ByteArray) =
            file.writeBytes(array)

    fun create() =
            file.create()

    fun delete(): Boolean =
            file.delete()

    companion object {
        fun from(path: String) =
                File(JavaFile(path))

        fun from(path: FilePath) =
                from(path.path)
    }
}

// TODO: Move to value class when mocking them with mockk works
class FilePath(val path: String)

/** Runs the given [block] in the context of a file with the given [fullPath]. */
fun <T> withFileAt(fullPath: FilePath, block: File.() -> T) =
        File.from(fullPath).block()

/** Shortcut to create a file and its parent directories. */
fun JavaFile.create() {
    Files.createDirectories(this.parentFile.toPath())
    Files.createFile(this.toPath())
}